import winshell
from os.path import join


class Desktop:

    def __init__(self, name, path, version, exe, description):
        self.name = name
        self.path = path
        self.version = version
        self.exe = exe
        self.description = description

    def create_shortcut(self):
        desktop = winshell.desktop()
        with winshell.shortcut(join(desktop, self.name + ' ' + self.version + ".lnk")) as shortcut:
            shortcut.path = self.path + self.version + '\\' + self.exe
            shortcut.icon_location = self.path + self.version + '\\' + self.exe, 0
            shortcut.description = self.description + self.version



import os

GH_API_ELONA = "https://api.github.com/repos/vichmartins/ElonaPlusCGXDownload/releases/latest"
GH_API_CGX = "https://api.github.com/repos/JianmengYu/ElonaPlusCustom-GX/releases/latest"
ELONA_FILE_NAME = 'Elona.zip'
CGX_FILE_NAME = 'Custom-GX.zip'
EXE = 'elonapluscgx.exe'
SHORTCUT_NAME = 'ElonaPlusCustom-GX'
INSTALL_PATH = os.getenv('APPDATA')

import download
import extract  
import desktop  
import move  
import cleanup  
import start_menu




import elonadl as elona

def main():
    print("downloading latest version of Elona+ to your %APPDATA% folder")
    elonaplus = elona.download.Download(file_name=elona.ELONA_FILE_NAME, link=elona.GH_API_ELONA, install_path=elona.INSTALL_PATH)
    elonaplus.create_file()
    elonaplus.get_file()

    print("downloading latest version of CustomGX variant.")
    cgx = elona.download.Download(file_name=elona.CGX_FILE_NAME, link=elona.GH_API_CGX, install_path=elona.INSTALL_PATH)
    cgx.create_file()
    cgx.get_file()

    print("extracting Elona+")
    e = elona.extract.Extract(elonaplus.get_name(), install_path=elona.INSTALL_PATH)
    e.extract_zip()

    print("extracting CustomGX")
    c = elona.extract.Extract(cgx.get_name(), install_path=elona.INSTALL_PATH)
    c.extract_zip()

    print("Moving files to the latest version")
    mover = elona.move.Move(source_path = cgx.get_install_path(), destination = e.find_latest_version())
    result = mover.execute()
    print(f"Operation {'successful' if result['success'] else 'failed'}")

    print("Renaming Elona+ directory")
    path = e.rename_directory()

    print("Creating Desktop Shortcut")
    version = e.scrape_version()
    d = elona.desktop.Desktop(name=elona.SHORTCUT_NAME, path=path, version=version, exe=elona.EXE, description=elona.SHORTCUT_NAME)
    d.create_shortcut()

    print("Cleaning up Downloaded Files")
    try:
        cleaner = elona.cleanup.Cleanup(main_game=elona.ELONA_FILE_NAME, mod=elona.CGX_FILE_NAME)
        cleaner.execute()
    except Exception as e:
        print(f"Error: {e}")

    print("Creating Start Menu Shortcut")
    menu = elona.start_menu.StartMenu(install_path=path + version + "\\" + elona.EXE, use_all_users=False, folder_name=elona.SHORTCUT_NAME, current_version=version)
    menu.create_shortcut()

    print('Setup Finished! Enjoy ElonaCustom-GX!')


if __name__ == "__main__":
    main()


from elonadl import SHORTCUT_NAME
from zipfile import ZipFile
from os import path, listdir, rename
from re import findall


class Extract:

    def __init__(self, file_name, install_path) -> None:
        self.file_name = file_name
        self.install_path = install_path

    def extract_zip(self):
        local = self.install_path
        c = path.join(local, self.file_name)

        with ZipFile(c) as z:
            z.extractall(local)

    def find_latest_version(self):
        t = listdir(path=self.install_path)
        n = "elonaplus"
        self.p = list()

        for x in t:
            h = findall(n + "[0-9].[0-9][0-9]", x)
            if h:
                self.p.append(x)

        if not self.p:
            print("No Elona+ directory found.")
            exit()
        
        result = path.join(self.install_path, max(self.p) + '\\')

        return result

    def scrape_version(self):
        t = listdir(path=self.install_path)
        n = SHORTCUT_NAME
        u = list()

        for x in t:
            h = findall(n + "[0-9].[0-9][0-9]", x)
            if h:
                u.append(x)

        return max(u)[18:22]
    
    def rename_directory(self):
        dir = max(self.p)
        rename(path.join(self.install_path, dir), 
               path.join(self.install_path + '\\' 
                    + SHORTCUT_NAME + dir[9:13]))
        
        return path.join(self.install_path 
                + '\\' + SHORTCUT_NAME)